# Specification

## Overview

The button provides feedback of its pressed and unpressed state.

This specification is an extension of [SDS Button Specification &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md)

## Use Cases

The button toggle enables an implementor to create a simple, but customizable button that toggles between a pressed and unpressed state.

## Non-goals

- Changing the text of the pressed state.

## Features

- The `pressed` state is reflected as an attribute on the host.
- The `pressed` state toggles the appropriate `aria` roles on the button.

## Prior Art/Examples

- [Salesforce Lightning Design System](https://www.lightningdesignsystem.com/components/buttons/)
- [Ant Design](https://ant.design/components/button/)
- [Atlassian](https://atlassian.design/components/button/examples)
- [Material Design](https://material.io/components/buttons)
- [Carbon](https://www.carbondesignsystem.com/components/button/usage/)
- [FAST](https://explore.fast.design/components/fast-button)
- [Morning Star](https://designsystem.morningstar.com/components/button/?version=3.1.4&tab=variations)
- [Polaris](https://polaris.shopify.com/components/actions/button)

---

## API

### Custom Element

This specification will apply to a SDS custom element but can also be implemented in a subsystem's ecosystem with the [subsystems namespace][namespace-service] used as the prefix to the custom elements definition:

1. `sds-button-toggle` - This is owned by the Salesforce Design System (SDS) Team.
2. `<namespace>-button-toggle` - This is owned by a subsystem built on SDS, e.g. `dxp-button-toggle`

### Anatomy and Appearance

The following DOM structure and named part attribute on the leaf node as documented is **required** to use the SDS button toggle CSS.

#### Diagram

[SDS Button - Diagram &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#diagram)

#### DOM Structure

[SDS Button - DOM Structure &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#dom-structure)

#### Properties & Attributes

[SDS Button - Properties & Attributes &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#properties--attributes)

| Attribute | Type   | Default Value | Description                                   |
| --------- | ------ | ------------- | --------------------------------------------- |
| `pressed` | `bool` | `false`       | Tracks if the button is pressed or unpressed. |

#### Events

[SDS Button - Events &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#events)

#### Styling Hooks

[SDS Button - Styling Hooks &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#styling-hooks)

Use the following Styling Hooks to customize the appearance of the button component. See [How to Use CSS in Your Web Component](https://github.com/salesforce-ux/salesforce-design-system/docs/subsystems/CSS_USAGE.md) for more details.

| Property Name                                    | CSS Property                             | Fallback Value | Description                                                                                                                          |
| ------------------------------------------------ | ---------------------------------------- | -------------- | ------------------------------------------------------------------------------------------------------------------------------------ |
| `--sds-c-button-color-background-pressed`        | `--sds-c-button-color-background`        | `transparent`  | Background color of button when pressed state is active.                                                                             |
| `--sds-c-button-color-background-pressed-hover`  | `--sds-c-button-color-background-hover`  | `transparent`  | Background color of button when pressed state is active and when a user interacts with button, but does not necessarily activate it. |
| `--sds-c-button-color-background-pressed-active` | `--sds-c-button-color-background-active` | `transparent`  | Background color of button when pressed state is active and when a user activates the button, toggling its state to unpressed.       |
| `--sds-c-button-text-color-pressed`              | `color`                                  | `#2b74d0`      | Text color of button when pressed state is active.                                                                                   |
| `--sds-c-button-text-color-pressed-hover`        | `--sds-c-button-text-color-hover`        | `#2b74d0`      | Text color of button when pressed state is active and when a user interacts with button, but does not necessarily activate it.       |
| `--sds-c-button-text-color-pressed-active`       | `--sds-c-button-text-color-active`       | `#2b74d0`      | Text color of button when pressed state is active and when a user activates the button, toggling its state to unpressed.             |
| `--sds-c-button-color-border-pressed`            | `border-color`                           | `transparent`  | Border color of button when pressed state is active.                                                                                 |
| `--sds-c-button-color-border-pressed-hover`      | `--sds-c-button-color-border-hover`      | `transparent`  | Border color of button when pressed state is active and when a user interacts with button, but does not necessarily activate it.     |
| `--sds-c-button-color-border-pressed-active`     | `--sds-c-button-color-border-active`     | `transparent`  | Border color of button when pressed state is active and when a user activates the button, toggling its state to unpressed.           |

---

## Behavior

### States & Interactions

[SDS Button - States & Interactions &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#states--interactions)

## Accessibility

### Keyboard Support

The `enter` and `space` keys will update the `pressed` state of the button toggle.

### `aria-pressed`

When the button toggle is in the `pressed` state, the child button element's `aria-pressed` state is also `true`. If the `pressed` state is not active, `aria-pressed` is set to `false`.

[SDS Button - Accessibility &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#accessibility)

## Internationalization

[SDS Button - Internationalization &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#internationalization)

## Test Plan

[SDS Button - Test Plan &rarr;](https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-components/src/button/button.spec.md#test-plan)

[button-spec]: https://html.spec.whatwg.org/multipage/form-elements.html#the-button-element
[anchor-spec]: https://html.spec.whatwg.org/multipage/text-level-semantics.html#the-a-element
[input-spec]: https://html.spec.whatwg.org/multipage/input.html#the-input-element
[input-button-spec]: https://html.spec.whatwg.org/multipage/input.html#button-state-(type=button)
[input-submit-spec]: https://html.spec.whatwg.org/multipage/input.html#submit-button-state-(type=submit)
[css-shadow-parts]: https://drafts.csswg.org/css-shadow-parts/
[styling-hooks-jump]: #styling-hooks
[namespace-service]: https://github.com/salesforce-ux/salesforce-design-system/blob/main/packages/sds-namespaces/src/namespaces.js
